==> py3-angr-9.0.6642p4.tgz: lib/python3.10/site-packages/angr/angrdb/serializers/funcs.py <==
# pylint:disable=unused-import
from ...knowledge_base import KnowledgeBase
from ...knowledge_plugins import FunctionManager, Function
from ..models import DbFunction, DbKnowledgeBase


class FunctionManagerSerializer:
    """
    Serialize/unserialize a function manager and its functions.
    """

    @staticmethod
    def dump(session, db_kb, func_manager):
        """

        :param session:
        :param DbKnowledgeBase db_kb:
        :param FunctionManager func_manager:
        :return:
        """

        # remove all existing functions
        session.query(DbFunction).filter_by(kb=db_kb).delete()

        for func in func_manager.values():
            db_func = DbFunction(
                kb=db_kb,
                addr=func.addr,
                blob=func.serialize(),
            )
            session.add(db_func)

    @staticmethod
    def load(session, db_kb, kb):
        """

        :param session:
        :param DbKnowledgeBase db_kb:
        :param KnowledgeBase kb:
        :return:                        A loaded function manager.
        """

        funcs = FunctionManager(kb)

        db_funcs = session.query(DbFunction).filter_by(kb=db_kb)
        all_func_addrs = set(map(lambda x: x[0], session.query(DbFunction.addr).filter_by(kb=db_kb)))

        for db_func in db_funcs:
            func = Function.parse(db_func.blob, function_manager=funcs, project=kb._project,
                                  all_func_addrs=all_func_addrs)
            funcs[func.addr] = func

        return funcs

==> py3-angr-9.2.85p2.tgz: lib/python3.12/site-packages/angr/angrdb/serializers/funcs.py <==
from typing import TYPE_CHECKING

from ...knowledge_plugins import FunctionManager, Function
from ..models import DbFunction

if TYPE_CHECKING:
    from angr.knowledge_base import KnowledgeBase
    from angr.angrdb.models import DbKnowledgeBase


class FunctionManagerSerializer:
    """
    Serialize/unserialize a function manager and its functions.
    """

    @staticmethod
    def dump(session, db_kb: "DbKnowledgeBase", func_manager: FunctionManager):
        """

        :param session:
        :param DbKnowledgeBase db_kb:
        :param FunctionManager func_manager:
        :return:
        """

        # remove all existing functions
        session.query(DbFunction).filter_by(kb=db_kb).delete()

        for func in func_manager.values():
            db_func = DbFunction(
                kb=db_kb,
                addr=func.addr,
                blob=func.serialize(),
            )
            session.add(db_func)

    @staticmethod
    def load(session, db_kb: "DbKnowledgeBase", kb: "KnowledgeBase"):
        """

        :param session:
        :param DbKnowledgeBase db_kb:
        :param KnowledgeBase kb:
        :return:                        A loaded function manager.
        """

        funcs = FunctionManager(kb)

        db_funcs = session.query(DbFunction).filter_by(kb=db_kb)
        all_func_addrs = set(map(lambda x: x[0], session.query(DbFunction.addr).filter_by(kb=db_kb)))

        for db_func in db_funcs:
            func = Function.parse(
                db_func.blob, function_manager=funcs, project=kb._project, all_func_addrs=all_func_addrs
            )
            funcs[func.addr] = func

        funcs.rebuild_callgraph()

        return funcs
